/*
   +----------------------------------------------------------------------+
   | PHP version 4.0                                                      |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.02 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available at through the world-wide-web at                           |
   | http://www.php.net/license/2_02.txt.                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors:                                                             |
   |                                                                      |
   +----------------------------------------------------------------------+
 */

#include "php.h"
#include "php_ini.h"
#include "php_iconv.h"
#include "../standard/php_smart_str.h"
#include <iconv.h>

function_entry iconv_functions[] = {
	PHP_FE(confirm_iconv_compiled,	NULL)		/* For testing, remove later. */
	PHP_FE(iconv, NULL)
	{NULL, NULL, NULL}	/* Must be the last line in iconv_functions[] */
};

zend_module_entry iconv_module_entry = {
	"iconv",
	iconv_functions,
	PHP_MINIT(iconv),
	PHP_MSHUTDOWN(iconv),
	NULL,
	NULL,
	PHP_MINFO(iconv),
	STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_ICONV
ZEND_GET_MODULE(iconv)
#endif

PHP_MINIT_FUNCTION(iconv)
{
	return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(iconv)
{
	return SUCCESS;
}

PHP_MINFO_FUNCTION(iconv)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "iconv support", "enabled");
	php_info_print_table_end();

}

/* Every user-visible function in PHP should document itself in the source */
/* {{{ proto string iconv(string from_charset, string to_charset, string str)
   Return a string to confirm that the module is compiled in */

#define ICONV_BUF_LEN 512

PHP_FUNCTION(iconv)
{
	iconv_t cd;

	zval **str;
	zval **to_charset;
	zval **from_charset;

	char *inbuf, *outbuf;
	char buf[ICONV_BUF_LEN];

	size_t res;
	size_t inlen, outlen;

	smart_str result = {0};
	size_t result_len;

	if (ZEND_NUM_ARGS() != 3 || zend_get_parameters_ex(3, &from_charset, &to_charset, &str) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

  	convert_to_string_ex(to_charset);
  	convert_to_string_ex(from_charset);
  	convert_to_string_ex(str);

	fprintf(stderr, "%s->%s\n", (*from_charset)->value.str.val, (*to_charset)->value.str.val);

  	cd = iconv_open((*to_charset)->value.str.val, (*from_charset)->value.str.val);

	if (!cd)
		RETURN_FALSE;

	inbuf  = (*str)->value.str.val;
  	inlen  = (*str)->value.str.len;

	result_len = 0;

	while (inlen) {
		outbuf = buf;
		outlen = ICONV_BUF_LEN;
		res = iconv(cd, &inbuf, &inlen, &outbuf, &outlen);

		if (((res == -1) && (errno == E2BIG))) {
			smart_str_appendl(&result, buf, ICONV_BUF_LEN - outlen);
			result_len += ICONV_BUF_LEN - outlen;
			errno = 0;
			continue;
		}

		if (res == -1) {
			switch (errno) {
			case EILSEQ:
				php_error(E_WARNING, "Invalid multibyte sequence in input buffer.");
				break;
			case EINVAL:
				php_error(E_WARNING, "Incomplete multibyte sequence in input buffer.");
				break;
			case E2BIG:
				php_error(E_WARNING, "Invalid output buffer size, kick the coder in the head.");
				break;
			default:
				php_error(E_WARNING, "Unknown error: other error!");
			}
			iconv_close(cd);
			RETURN_FALSE;
		}
		if (inlen == 0) {
			smart_str_appendl(&result, buf, ICONV_BUF_LEN - outlen);
			smart_str_0(&result);
			iconv_close(cd);
			result_len += ICONV_BUF_LEN - outlen;
			RETURN_STRINGL(result.c, result_len, 0);
		}

		php_error(E_WARNING, "Unknown error: inlen not 0!");
		iconv_close(cd);
		RETURN_FALSE;
	}
}

/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 */
